/*
 * PROIECT PM
 * Diaconu Virgiliu-Radu, 335cc
 * HID based Alarm
 */
 

#include <avr\io.h>
#include <avr\interrupt.h>

#define F_CPU 16000000
#include <util/delay.h>
#include <stdlib.h>

/* definim senzorii de la care primeste infomartii alarma */
#define Switch1 1
#define Switch2 2
#define IR1 3
#define IR2	4

/* definim porturile unde sunt conectati senzorii */
#define PORT_S1 5
#define PORT_S2 6
#define PORT_IR1 2
#define PORT_IR2 0

/* folosite pt. a modifica sunet buzzer */
#define COUNT_MAX 200
#define COUNT_MIN 50


int val;
uint8_t channel = 0;



/* functie care initializeaza buzzerul */
void initBuzz()
{
	DDRD |= _BV(PD5);						// punem pe output pinul asociat timer-ului
	TCCR1A |= _BV(COM1A0) | _BV(WGM10);
	TCCR1B |= _BV(WGM12);			 		// 64kHz
	TIMSK |= OCIE1A;
	OCR1A = COUNT_MIN; 					
}


/* functie care opreste buzzerul */
void stopBuzz()
{
	TCCR1B = 0;
}


/* functie folosita pt. a genera delay-uri mari */
void doDelay(unsigned int delay)
{
	int ii;
	for(ii=0; ii < delay/100; ii++)
		_delay_ms(100);
}


/* functie care aplica diverse tensiuni pe buzzer, astfel incat 
 * acesta sa emita sunete diferite */
void buzz()
{
	int count;
	
	// pornesc timerul
	TCCR1B |= _BV(CS12);
	
	for(count = 0; count < 3; count++)
	{
		OCR1A = COUNT_MAX;
		doDelay(2000);
		OCR1A = COUNT_MIN;
		doDelay(2000);
	}
}


/* functie care citeste de pe un canal din portul A folosindu-se
 * de convertorul analog-digital */
int ADC_get(uint8_t channel)
{	
	//activam rezistenta de pull-up pe pinul corespunzator canalului
	PORTA |= _BV(channel);
	
	//setati canalul de pe care vreti sa faceti citirea
	ADMUX &= 0xE0;
	ADMUX |= channel;
	ADCSRA |= _BV(ADEN)|_BV(ADSC);
	//asteptati sa se termine o conversie
	while(!(ADCSRA & (1<<ADIF)));
	//returnati rezultatul conversiei
	return ADC;
}


/* functie care initializeaza ADC-ul */
void ADC_init(void)
{
	// AVCC cu condesator pe AREF;
	ADMUX = (1<<REFS0);
	// activeaza ADC si intreruperea de conversie. Prescaler de ceas la maximum
	ADCSRA = (1<<ADEN) | (1<<ADATE) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);
}



int main(void)
{	
	int s1, s2, ir1, ir2;
	
	// seteaza pe 1 bitul I din SREG, activnd ntreruperile globale
	sei();
	
	// realizam initializari
	ADC_init();
	initBuzz();
	
	while (1)
	{
			stopBuzz();
			
			s1 = ADC_get(PORT_S1);
			s2 = ADC_get(PORT_S2);
			
			ir1 = ADC_get(PORT_IR1);
			ir2 = ADC_get(PORT_IR2);
			
			// un switch a fost apasat
			if (s1 <= 10 || s2 <= 10)
				buzz();
			
			// unul dintre senzorii de prezenta a sesizat miscare 
			if (ir1 >= 700 || ir2 >= 700)
				buzz();
	}
		
	return 0;
}